==> traffic-0.5.0dev.tar.gz: traffic-0.5.0dev/traffic/test/test_lib.py <==
##
# .test.test_libtraffic
##
import time
import os
import array
from .. import lib as libtraffic
from ..libresource import sockets as sockets_buffer

def test_module_protocol(test):
	test.fail_if_not_hasattr(libtraffic, "Transit")
	test.fail_if_not_hasattr(libtraffic, "Point")
	test.fail_if_not_hasattr(libtraffic, "Traffic")
	test.fail_if_not_hasattr(libtraffic, "Octets")
	test.fail_if_not_hasattr(libtraffic, "Sockets")
	test.fail_if_not_hasattr(libtraffic, "Moments")
	test.fail_if_not_hasattr(libtraffic, "Exits")

	test.fail_if_not_subclass(libtraffic.Point, libtraffic.Transit)
	test.fail_if_not_subclass(libtraffic.Traffic, libtraffic.Point)
	test.fail_if_not_subclass(libtraffic.Octets, libtraffic.Point)
	test.fail_if_not_subclass(libtraffic.Sockets, libtraffic.Point)
	test.fail_if_not_subclass(libtraffic.Exits, libtraffic.Transit)
	test.fail_if_not_subclass(libtraffic.Moments, libtraffic.Transit)
	types = (
		libtraffic.Transit, libtraffic.Exits,
		libtraffic.Point, libtraffic.Traffic,
		libtraffic.Octets, libtraffic.Sockets,
		libtraffic.Moments,
	)
	for x in types:
		test.fail_if_not_hasattr(x, "transfer_event_id")
		test.fail_if_not_hasattr(x, "exhaust_event_id")
		test.fail_if_not_hasattr(x, "terminate_event_id")
		test.fail_if_not_subclass(x.TransitError, libtraffic.TransitError)
		test.fail_if_not_subclass(x.TerminatedError, libtraffic.TransitError)
		test.fail_if_not_subclass(x.ResourceError, libtraffic.TransitError)

def test_subtyping(test):
	types = (
		libtraffic.Exits, libtraffic.Traffic,
		libtraffic.Octets, libtraffic.Sockets,
		libtraffic.Moments,
	)
	for x in types:
		with test.trap(TypeError) as t:
			# Transit types extend the storage internally.
			# Discourage subtyping.
			# XXX: Transit can still be subclassed?
			class Foo(x):
				pass

def test_traffic_termination(test):
	'traffic termination sequence with no transits'
	t = libtraffic.Traffic()
	test.fail_if(t.terminated)
	t.terminate()
	test.fail_if(t.terminated) # it hasn't completed termination.
	test.fail_if_not(t.terminating)
	with t:
		test.fail_if_not(t.terminating)
	# one cycle in..
	test.fail_if_not_raised(t.TerminatedError, t.__enter__)
	test.fail_if_not(t.terminated)

def test_traffic_exit(test):
	'context manager to terminate on exit'
	class LocalExc(Exception):
		pass
	T = libtraffic.Traffic()
	with test.trap(LocalExc) as trap:
		with T:
			raise LocalExc("foo")
	test.fail_if(T.terminated)

def test_traffic_force(test):
	T = libtraffic.Traffic()
	# once per second
	# this gives T a kevent
	T.force()
	start = time.time()
	with T:
		period = time.time() - start
		test.fail_if_not_empty(T.transfer())
	# rather aribtrary number...
	test.fail_if_greater_than(period, 0.3)
	T.terminate()

def traffic_nothing(test, T):
	'check idle traffic'
	with T:
		test.fail_if_not_empty(T.transfer())

def traffic_termination(test, T):
	T.terminate()
	# traffic termination cascades to transits
	with T:
		transits = set(T.transfer())
	test.fail_if_not(T.terminated)
	for x in transits:
		test.fail_if_not(x.terminated)
	return transits

def test_moments(test):
	# this ends up testing kqueue and gettimeofday more than
	# I would like, but I'm not sure if there's a better way.
	import time
	events = []
	m = libtraffic.Moments(2)
	T = libtraffic.Traffic()
	T.acquire(m)
	start = time.time()
	while time.time() - start < 1.3:
		with T:
			transits = list(T.transfer())
			if transits:
				test.fail_if_not_exact(m, transits[0])
				events = list(m.events())
				test.fail_if_not_equal([m.transfer_event_id], events)
	m.terminate()
	with T:
		t = list(T.transfer())[0]
		test.fail_if_not_equal(list(t.events()), [t.terminate_event_id])
	test.fail_if_not(m.terminated)
	with T:
		# terminated, should not produce event.
		test.fail_if_not_empty(T.transfer())

def test_exits(test):
	import signal, os, time
	child = os.fork()
	if child == 0:
		# child proc
		while True:
			time.sleep(10)
	ck = libtraffic.Exits(child)
	T = libtraffic.Traffic()
	T.acquire(ck)
	with T:
		pass
	os.kill(child, signal.SIGKILL)
	status = os.waitpid(child, 0)
	while True:
		with T:
			l = list(T.transfer())
			if l:
				x = l[0]
				test.fail_if_not_equal(tuple(x.events())[0], x.terminate_event_id)
				break
	traffic_termination(test, T)

def connection_transfer(test, T, spairs):
	io1, io2 = spairs # connected io pairs
	data = {
		io1[1]: b'foobar',
		io2[1]: b'barfoo',
	}
	transits = io1 + io2
	for x in transits:
		x.acquire(data.get(x) or bytearray(128))
		T.acquire(x)

	# do writes
	with T:
		for x in T.transfer():
			test.fail_if_not_equal(tuple(x.events()), (x.transfer_event_id, x.exhaust_event_id))

	# do reads
	with T:
		for x in T.transfer():
			for y in x.events():
				test.fail_if_not_equal(y, x.transfer_event_id)
				xfer = bytes(x.transfer())
				if x is io2[0]:
					test.fail_if_not_equal(xfer, data[io1[1]])
				elif x is io1[0]:
					test.fail_if_not_equal(xfer, data[io2[1]])
	traffic_nothing(test, T)

def remote_termination(test, T, terminating, receiver):
	'checked that remote termination is signalling termination on a receiver'
	traffic_nothing(test, T)
	terminating.terminate()
	transits = {terminating, receiver}
	with T:
		for x in T.transfer():
			test.fail_if_not_in(transits)
			for y in x.events():
				test.fail_if_not_equal(x.terminate_event_id)

def new_socketpair_transfer(test, T):
	spairs = libtraffic.Octets.connect_stream_socketpair()
	return connection_transfer(test, T, spairs)

def test_socketpair_transfer(test):
	T = libtraffic.Traffic()
	new_socketpair_transfer(test, T)
	transits = traffic_termination(test, T)

def new_pipe_transfer(test, T):
	r1, w2 = libtraffic.Octets.connect_pipe()
	r2, w1 = libtraffic.Octets.connect_pipe()
	connection_transfer(test, T, ((r1, w1), (r2, w2)))
	return {r1,r2,w1,w2}

def test_pipe(test):
	T = libtraffic.Traffic()
	created = new_pipe_transfer(test, T)
	traffic_termination(test, T)
	for x in created:
		test.fail_if_not_equal(True, x.terminated)

def test_file(test):
	import os, tempfile, shutil
	tmpdir = tempfile.mkdtemp()
	try:
		path = os.path.join(tmpdir, "wfile")
		apath = path.encode('ascii')
		count = 0
		wrote_data = []
		thedata = b'\xF1'*128

		T = libtraffic.Traffic()
		wr = libtraffic.Octets.connect_file_append(apath)
		T.acquire(wr)
		wr.acquire(thedata)
		test.fail_if_not_exact(wr.resource, thedata)
		while not wr.terminated:
			with T:
				for x in T.transfer():
					for e in x.events():
						if e == x.transfer_event_id:
							pass
						elif e == x.exhaust_event_id:
							wrote_data.append(thedata)
							x.acquire(thedata)
							count += 1
							if count == 16:
								x.terminate()
		os.chmod(apath, 0o700)
		# now read it back in.
		data = []
		rd = libtraffic.Octets.connect_file(apath)
		T.acquire(rd)
		rd.acquire(bytearray(128))
		while not rd.terminated:
			with T:
				for x in T.transfer():
					for e in x.events():
						if e == x.transfer_event_id:
							data.append(x.transfer().tobytes())
							pass
							if sum(map(len,data)) == len(thedata):
								x.terminate()
						elif e == x.exhaust_event_id and not x.terminating:
							x.acquire(bytearray(128))
	finally:
		shutil.rmtree(tmpdir)

def showevents(T):
	transits = tuple(T.transfer())
	if not transits:
		print("no events")
	else:
		for x in transits:
			print(x, x.identity(), x.opposite.identity() if hasattr(x, 'opposite') else None, ":", tuple(x.events()))

def test_sockets_stream_local(test):
	import os, tempfile, shutil
	tmpdir = tempfile.mkdtemp()
	afilepath = os.path.join(tmpdir, 'server_fss')
	filepath = afilepath.encode('ascii')
	try:
		sf = libtraffic.Sockets.accept_stream_local(filepath)
		sf.acquire(sockets_buffer(8))
		os.chmod(filepath, 0o700)
		T = libtraffic.Traffic()
		T.acquire(sf)
		with T:
			pass
		test.fail_if_not_equal(filepath, sf.identity()[0])
		client = libtraffic.Octets.connect_stream_local(filepath)
		T.acquire(client[0])
		T.acquire(client[1])
		accepted = list()
		with T:
			for x in sf.resource[slice(*sf.transfer())]:
				accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
		with T:
			for x in sf.resource[slice(*sf.transfer())]:
				accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
		with T:
			pass
		test.fail_if_not_equal(filepath, client[1].identity()[0])
		with T:
			pass
	finally:
		shutil.rmtree(tmpdir)

def test_sockets_stream_ip4(test):
	sf = libtraffic.Sockets.accept_stream_ip4(interface = ('127.0.0.1', 0))
	sf.acquire(sockets_buffer(8))
	T = libtraffic.Traffic()
	T.acquire(sf)
	host, port = sf.identity()
	test.fail_if_not_equal(host, '127.0.0.1')
	client = libtraffic.Octets.connect_stream_ip4(('127.0.0.1', port))
	with T:
		accepted = list()
		for x in sf.resource[slice(*sf.transfer())]:
			accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
	with T:
		for x in sf.resource[slice(*sf.transfer())]:
			accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
	client[1].terminate()
	with T:
		pass
	with T:
		pass

def test_sockets_stream_ip6(test):
	sf = libtraffic.Sockets.accept_stream_ip6(('::1', 0))
	sf.acquire(sockets_buffer(8))
	T = libtraffic.Traffic()
	T.acquire(sf)
	host, port = sf.identity()
	test.fail_if_not_equal(host, '::1')
	client = tuple(map(T.acquire, libtraffic.Octets.connect_stream_ip6((host,port))))

if __name__ == '__main__':
	from dev import libtest; libtest.execmodule()

==> traffic-0.6.0dev.tar.gz: traffic-0.6.0dev/traffic/test/test_lib.py <==
##
# .test.test_libtraffic
##
import time
import os
import array
from .. import lib as libtraffic
from ..libresource import sockets as sockets_buffer

def test_module_protocol(test):
	test.fail_if_not_hasattr(libtraffic, "Transit")
	test.fail_if_not_hasattr(libtraffic, "Point")
	test.fail_if_not_hasattr(libtraffic, "Traffic")
	test.fail_if_not_hasattr(libtraffic, "Octets")
	test.fail_if_not_hasattr(libtraffic, "Sockets")
	test.fail_if_not_hasattr(libtraffic, "Moments")
	test.fail_if_not_hasattr(libtraffic, "Exits")

	test.fail_if_not_subclass(libtraffic.Point, libtraffic.Transit)
	test.fail_if_not_subclass(libtraffic.Traffic, libtraffic.Point)
	test.fail_if_not_subclass(libtraffic.Octets, libtraffic.Point)
	test.fail_if_not_subclass(libtraffic.Sockets, libtraffic.Point)
	test.fail_if_not_subclass(libtraffic.Exits, libtraffic.Transit)
	test.fail_if_not_subclass(libtraffic.Moments, libtraffic.Transit)
	types = (
		libtraffic.Transit, libtraffic.Exits,
		libtraffic.Point, libtraffic.Traffic,
		libtraffic.Octets, libtraffic.Sockets,
		libtraffic.Moments,
	)
	for x in types:
		test.fail_if_not_hasattr(x, "transfer_event_id")
		test.fail_if_not_hasattr(x, "exhaust_event_id")
		test.fail_if_not_hasattr(x, "terminate_event_id")
		test.fail_if_not_subclass(x.TransitError, libtraffic.TransitError)
		test.fail_if_not_subclass(x.TerminatedError, libtraffic.TransitError)
		test.fail_if_not_subclass(x.ResourceError, libtraffic.TransitError)

def test_subtyping(test):
	types = (
		libtraffic.Exits, libtraffic.Traffic,
		libtraffic.Octets, libtraffic.Sockets,
		libtraffic.Moments,
	)
	for x in types:
		with test.trap(TypeError) as t:
			# Transit types extend the storage internally.
			# Discourage subtyping.
			# XXX: Transit can still be subclassed?
			class Foo(x):
				pass

def test_traffic_termination(test):
	'traffic termination sequence with no transits'
	t = libtraffic.Traffic()
	test.fail_if(t.terminated)
	t.terminate()
	test.fail_if(t.terminated) # it hasn't completed termination.
	test.fail_if_not(t.terminating)
	with t:
		test.fail_if_not(t.terminating)
	# one cycle in..
	test.fail_if_not_raised(t.TerminatedError, t.__enter__)
	test.fail_if_not(t.terminated)

def test_traffic_exit(test):
	'context manager to terminate on exit'
	class LocalExc(Exception):
		pass
	T = libtraffic.Traffic()
	with test.trap(LocalExc) as trap:
		with T:
			raise LocalExc("foo")
	test.fail_if(T.terminated)

def test_traffic_force(test):
	T = libtraffic.Traffic()
	# once per second
	# this gives T a kevent
	T.force()
	start = time.time()
	with T:
		period = time.time() - start
		test.fail_if_not_empty(T.transfer())
	# rather aribtrary number...
	test.fail_if_greater_than(period, 0.3)
	T.terminate()

def traffic_nothing(test, T):
	'check idle traffic'
	with T:
		test.fail_if_not_empty(T.transfer())

def traffic_termination(test, T):
	T.terminate()
	# traffic termination cascades to transits
	with T:
		transits = set(T.transfer())
	test.fail_if_not(T.terminated)
	for x in transits:
		test.fail_if_not(x.terminated)
	return transits

def test_moments(test):
	# this ends up testing kqueue and gettimeofday more than
	# I would like, but I'm not sure if there's a better way.
	import time
	events = []
	m = libtraffic.Moments(2)
	T = libtraffic.Traffic()
	T.acquire(m)
	start = time.time()
	while time.time() - start < 1.3:
		with T:
			transits = list(T.transfer())
			if transits:
				test.fail_if_not_exact(m, transits[0])
				events = list(m.events())
				test.fail_if_not_equal([m.transfer_event_id], events)
	m.terminate()
	with T:
		t = list(T.transfer())[0]
		test.fail_if_not_equal(list(t.events()), [t.terminate_event_id])
	test.fail_if_not(m.terminated)
	with T:
		# terminated, should not produce event.
		test.fail_if_not_empty(T.transfer())

def test_exits(test):
	import signal, os, time
	child = os.fork()
	if child == 0:
		# child proc
		while True:
			time.sleep(10)
	ck = libtraffic.Exits(child)
	T = libtraffic.Traffic()
	T.acquire(ck)
	with T:
		pass
	os.kill(child, signal.SIGKILL)
	status = os.waitpid(child, 0)
	while True:
		with T:
			l = list(T.transfer())
			if l:
				x = l[0]
				test.fail_if_not_equal(tuple(x.events())[0], x.terminate_event_id)
				break
	traffic_termination(test, T)

def connection_transfer(test, T, spairs):
	io1, io2 = spairs # connected io pairs
	data = {
		io1[1]: b'foobar',
		io2[1]: b'barfoo',
	}
	transits = io1 + io2
	for x in transits:
		x.acquire(data.get(x) or bytearray(128))
		T.acquire(x)

	# do writes
	with T:
		for x in T.transfer():
			test.fail_if_not_equal(tuple(x.events()), (x.transfer_event_id, x.exhaust_event_id))

	# do reads
	with T:
		for x in T.transfer():
			for y in x.events():
				test.fail_if_not_equal(y, x.transfer_event_id)
				xfer = bytes(x.resource[x.transfer()])
				if x is io2[0]:
					test.fail_if_not_equal(xfer, data[io1[1]])
				elif x is io1[0]:
					test.fail_if_not_equal(xfer, data[io2[1]])
	traffic_nothing(test, T)

def remote_termination(test, T, terminating, receiver):
	'checked that remote termination is signalling termination on a receiver'
	traffic_nothing(test, T)
	terminating.terminate()
	transits = {terminating, receiver}
	with T:
		for x in T.transfer():
			test.fail_if_not_in(transits)
			for y in x.events():
				test.fail_if_not_equal(x.terminate_event_id)

def new_socketpair_transfer(test, T):
	spairs = libtraffic.Octets.connect_stream_socketpair()
	return connection_transfer(test, T, spairs)

def test_socketpair_transfer(test):
	T = libtraffic.Traffic()
	new_socketpair_transfer(test, T)
	transits = traffic_termination(test, T)

def new_pipe_transfer(test, T):
	r1, w2 = libtraffic.Octets.connect_pipe()
	r2, w1 = libtraffic.Octets.connect_pipe()
	connection_transfer(test, T, ((r1, w1), (r2, w2)))
	return {r1,r2,w1,w2}

def test_pipe(test):
	T = libtraffic.Traffic()
	created = new_pipe_transfer(test, T)
	traffic_termination(test, T)
	for x in created:
		test.fail_if_not_equal(True, x.terminated)

def test_file(test):
	import os, tempfile, shutil
	tmpdir = tempfile.mkdtemp()
	try:
		path = os.path.join(tmpdir, "wfile")
		apath = path.encode('ascii')
		count = 0
		wrote_data = []
		thedata = b'\xF1'*128

		T = libtraffic.Traffic()
		wr = libtraffic.Octets.connect_file_append(apath)
		T.acquire(wr)
		wr.acquire(thedata)
		test.fail_if_not_exact(wr.resource, thedata)
		while not wr.terminated:
			with T:
				for x in T.transfer():
					for e in x.events():
						if e == x.transfer_event_id:
							pass
						elif e == x.exhaust_event_id:
							wrote_data.append(thedata)
							x.acquire(thedata)
							count += 1
							if count == 16:
								x.terminate()
		os.chmod(apath, 0o700)
		# now read it back in.
		data = []
		rd = libtraffic.Octets.connect_file(apath)
		T.acquire(rd)
		rd.acquire(bytearray(128))
		while not rd.terminated:
			with T:
				for x in T.transfer():
					for e in x.events():
						if e == x.transfer_event_id:
							data.append(bytes(x.resource[x.transfer()]))
							pass
							if sum(map(len,data)) == len(thedata):
								x.terminate()
						elif e == x.exhaust_event_id and not x.terminating:
							x.acquire(bytearray(128))
	finally:
		shutil.rmtree(tmpdir)

def showevents(T):
	transits = tuple(T.transfer())
	if not transits:
		print("no events")
	else:
		for x in transits:
			print(x, x.identity(), x.opposite.identity() if hasattr(x, 'opposite') else None, ":", tuple(x.events()))

def test_sockets_stream_local(test):
	import os, tempfile, shutil
	tmpdir = tempfile.mkdtemp()
	afilepath = os.path.join(tmpdir, 'server_fss')
	filepath = afilepath.encode('ascii')
	try:
		sf = libtraffic.Sockets.accept_stream_local(filepath)
		sf.acquire(sockets_buffer(8))
		os.chmod(filepath, 0o700)
		T = libtraffic.Traffic()
		T.acquire(sf)
		with T:
			pass
		test.fail_if_not_equal(filepath, sf.identity()[0])
		client = libtraffic.Octets.connect_stream_local(filepath)
		T.acquire(client[0])
		T.acquire(client[1])
		accepted = list()
		with T:
			for x in sf.resource[sf.transfer()]:
				accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
		with T:
			for x in sf.resource[sf.transfer()]:
				accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
		with T:
			pass
		test.fail_if_not_equal(filepath, client[1].identity()[0])
		with T:
			pass
	finally:
		shutil.rmtree(tmpdir)

def test_sockets_stream_ip4(test):
	sf = libtraffic.Sockets.accept_stream_ip4(interface = ('127.0.0.1', 0))
	sf.acquire(sockets_buffer(8))
	T = libtraffic.Traffic()
	T.acquire(sf)
	host, port = sf.identity()
	test.fail_if_not_equal(host, '127.0.0.1')
	client = libtraffic.Octets.connect_stream_ip4(('127.0.0.1', port))
	with T:
		accepted = list()
		for x in sf.resource[sf.transfer()]:
			accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
	with T:
		for x in sf.resource[sf.transfer()]:
			accepted.append(tuple(map(T.acquire, libtraffic.Octets.connect_stream_socket(x))))
	client[1].terminate()
	with T:
		pass
	with T:
		pass

def test_sockets_stream_ip6(test):
	sf = libtraffic.Sockets.accept_stream_ip6(('::1', 0))
	sf.acquire(sockets_buffer(8))
	T = libtraffic.Traffic()
	T.acquire(sf)
	host, port = sf.identity()
	test.fail_if_not_equal(host, '::1')
	client = tuple(map(T.acquire, libtraffic.Octets.connect_stream_ip6((host,port))))

if __name__ == '__main__':
	from dev import libtest; libtest.execmodule()
